/*
 * FILE NAME cpmodem_shim.h
 *
 * BRIEF MODULE DESCRIPTION
 *  Frankendriver - USB to ethernet, ip or PPP controlled via a block driver.
 *
 *  Author: CradlePoint Technology, Inc.  <source@cradlepoint.com>
 *          Ben Kendall <benk@cradlepoint.com>
 *          Cory Atkin <catkin@cradlepoint.com>
 *
 * Copyright 2012, CradlePoint Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to:
 *  Free Software Foundation
 *  51 Franklin Street, Fifth Floor
 *  Boston, MA  02111-1301  USA
 */

#ifndef __PKG_CPLKM_H__
#define __PKG_CPLKM_H__

#include <linux/if_ether.h>
#include <linux/types.h> 

#ifndef __KERNEL__
#define list_head list
#endif

// OR'd mask bits for setting the log mask
#define LOG_DEBUG_LEVEL_ERROR	(1 << 0)
#define LOG_DEBUG_LEVEL_WARN	(1 << 1)
#define LOG_DEBUG_LEVEL_INFO	(1 << 2)
#define LOG_DEBUG_LEVEL_TRACE	(1 << 3)
#define LOG_DEBUG_LEVEL_PRINTF	(1 << 4)

// LOG_DEBUG_LEVEL_ASSERT is always on 

//state of the usb device as a whole
typedef enum{
    CP_LKM_USB_INIT =    0,  //ioctl created the object, but we haven't gotten the probe yet from USB
    CP_LKM_USB_CTRL =    1,  //got probe from usb, can now pass ctrl pkts between USB device and the app space driver
    CP_LKM_USB_ACTIVE =  2,  //ioctl told us the data connection is now active
    CP_LKM_USB_DEAD =    3,  //Got disconnect from usb, can't use this bad boy anymore (waiting for ioctl to clean us up)
	CP_LKM_USB_INVALID = 0xff
}cp_lkm_usb_state_t;

//state of the usb device data processing state machine
typedef enum{
	USB_PROCESS_STATE_IDLE = 0,  //no softirq processing going on
	USB_PROCESS_STATE_ACTIVE,    //softirq machine processing 
	USB_PROCESS_STATE_PAUSED     //machine paused (link, unlink, unplug)
}cp_lkm_usb_process_state_t;

//These must match the cpusb_wrapper_t enum in pkgCPUSB.h
typedef enum
{
    CP_LKM_WRAPPER_NONE = 0,
    CP_LKM_WRAPPER_ASIX,
    CP_LKM_WRAPPER_ASIX_88179,
    CP_LKM_WRAPPER_LG,
    CP_LKM_WRAPPER_DIRECT_IP,
    CP_LKM_WRAPPER_MSRNDIS,
    CP_LKM_WRAPPER_PEGASUS,
    CP_LKM_WRAPPER_NCM,
	CP_LKM_WRAPPER_QMAP,          //New Qualcomm muxing and aggregation protocol
    CP_LKM_WRAPPER_NUM_WRAPPERS //NOTE: this one needs to always be last
}cp_lkm_wrapper_t;

#define CP_LKM_IOCTL_MAGIC 0xC1	// pick something not in /usr/src/linux/Documentation/ioctl/ioctl-number.txt
#define CP_LKM_IOCTL_USB_PLUG_INTF 		_IOW(CP_LKM_IOCTL_MAGIC, 0, struct cp_lkm_usb_plug_intf)
#define CP_LKM_IOCTL_USB_OPEN_INTF		_IOW(CP_LKM_IOCTL_MAGIC, 1, struct cp_lkm_usb_open_intf)
#define CP_LKM_IOCTL_USB_CLOSE_INTF		_IOW(CP_LKM_IOCTL_MAGIC, 2, struct cp_lkm_usb_close_intf)
#define CP_LKM_IOCTL_USB_UNPLUG_INTF	_IOW(CP_LKM_IOCTL_MAGIC, 3, struct cp_lkm_usb_unplug_intf)
#define CP_LKM_IOCTL_USB_EP_ACTION  	_IOW(CP_LKM_IOCTL_MAGIC, 4, struct cp_lkm_usb_ep_action)
#define CP_LKM_IOCTL_USB_PM_LINK		_IOW(CP_LKM_IOCTL_MAGIC, 5, struct cp_lkm_usb_pm_link)
#define CP_LKM_IOCTL_USB_SET_WRAPPER	_IOW(CP_LKM_IOCTL_MAGIC, 6, struct cp_lkm_usb_set_wrapper)
#define CP_LKM_IOCTL_USB_SET_MUX_ID		_IOW(CP_LKM_IOCTL_MAGIC, 7, struct cp_lkm_usb_set_mux_id)
#define CP_LKM_IOCTL_USB_IS_ALIVE_INTF	_IOW(CP_LKM_IOCTL_MAGIC, 8, struct cp_lkm_usb_is_alive_intf)

#define CP_LKM_IOCTL_PM_ATTACH			_IOWR(CP_LKM_IOCTL_MAGIC, 10, struct cp_lkm_pm_attach_ioctl)
#define CP_LKM_IOCTL_PM_DETACH			_IOW(CP_LKM_IOCTL_MAGIC, 11, struct cp_lkm_pm_detach_ioctl)
#define CP_LKM_IOCTL_PM_ACTIVATE		_IOW(CP_LKM_IOCTL_MAGIC, 12, struct cp_lkm_pm_activate_deactivate_ioctl)
#define CP_LKM_IOCTL_PM_DEACTIVATE		_IOW(CP_LKM_IOCTL_MAGIC, 13, struct cp_lkm_pm_activate_deactivate_ioctl)

#define CP_LKM_IOCTL_PM_ADD_FILTER		_IOW(CP_LKM_IOCTL_MAGIC, 14, struct cp_lkm_pm_add_filter_ioctl)

#define CP_LKM_IOCTL_SET_LOG_LEVEL 		_IOW(CP_LKM_IOCTL_MAGIC, 20, int)

// protocol manager types - keep in sync with protocol_manager_type_t in pkgProtocolManager.h
typedef enum {
	CP_LKM_PM_TYPE_PPP_CLIENT,//Default for modems
    CP_LKM_PM_TYPE_PPP_SERVER,
	CP_LKM_PM_TYPE_IP_STATIC,
	CP_LKM_PM_TYPE_IP_DHCP,
	CP_LKM_PM_TYPE_ETHERNET_STATIC,
    CP_LKM_PM_TYPE_ETHERNET_STATIC_NOARP,
	CP_LKM_PM_TYPE_ETHERNET_DHCP,
	CP_LKM_PM_TYPE_PPPOE_STATIC,
	CP_LKM_PM_TYPE_PPPOE_DHCP,
	CP_LKM_PM_TYPE_PPTP_STATIC,
	CP_LKM_PM_TYPE_PPTP_DHCP,
	CP_LKM_PM_TYPE_L2TP_STATIC,
	CP_LKM_PM_TYPE_L2TP_DHCP,
	CP_LKM_PM_TYPE_TUN_TAP_STATIC,
	CP_LKM_PM_TYPE_TUN_TAP_DHCP,
	CP_LKM_PM_TYPE_MAX
} cp_lkm_pm_type_t;

typedef enum
{
    CP_LKM_USB_CMD_DATA_SEND = 0,  //send a usb data pkt to shim
    CP_LKM_USB_CMD_DATA_RECV,      //recv a usb data pkt from shim
    CP_LKM_USB_CMD_CTRL_SEND,      //send a usb ctrl pkt to shim
    CP_LKM_USB_CMD_CTRL_RECV,       //recv a usb ctrl pkt from the shim
}cp_lkm_usb_cmd_t;

#define CP_LKM_MAX_IF_NAME 16 // really 15 + NULL
struct cp_lkm_pm_attach_ioctl {
	unsigned int uid;
	cp_lkm_pm_type_t type;
	char *name;
	unsigned char mac[ETH_ALEN];
};

struct cp_lkm_pm_detach_ioctl {
	unsigned int uid;
	cp_lkm_pm_type_t type;
};

struct cp_lkm_pm_activate_deactivate_ioctl {
	unsigned int uid;
	cp_lkm_pm_type_t type;
};

// data structures to implement shim traffic filter
typedef enum {
	CP_LKM_PM_FILTER_TYPE_NONE = 0,
	CP_LKM_PM_FILTER_TYPE_IP_SRC_WAN_SUBNET_INCLUDE, // allow egress packets src'd from the wan subnet
	CP_LKM_PM_FILTER_TYPE_IP_SRC_SUBNET_INCLUDE, // allow egress packets src'd from the specified subnet
	CP_LKM_PM_FILTER_TYPE_MAX
} cp_lkm_pm_filter_type_t;


struct ip_src_subnet_include {
	unsigned int ipv4_addr;
	unsigned int ipv4_mask;
};

struct cp_lkm_pm_filter {
	cp_lkm_pm_filter_type_t type;
	union {
		// filter parameter structures based on type
		struct ip_src_subnet_include subnet;
	};
	struct list_head list;
};

struct cp_lkm_pm_add_filter_ioctl {
	unsigned int uid;
	struct cp_lkm_pm_filter filter;
};

// CP_LKM_PM commands
enum {
	CP_LKM_PM_LINK_DOWN = 0
};

enum
{
    CP_LKM_STATUS_OK = 0,
    CP_LKM_STATUS_TIMEOUT,
    CP_LKM_STATUS_ERROR
};

struct cp_lkm_msg_hdr {
	int instance_id;
	int cmd;
    int arg1;    //cmd specific args
    int arg2;    //cmd specific args
    int status;
	int len; // not including this structure
};

//struct defs for each command
struct cp_lkm_usb_plug_intf
{
    int bus;
    int addr;
    int vid;
    int pid;
    int ep_in;         //data bulk in ep number
    int ep_out;        //data bulk out ep number
    int intf_num;
    int alt_intf_num;
    int unique_id;
    int max_transfer_size;
	int feature_flags;
	int clone_num;  
	int mux_id;  
};

#define CP_LKM_FEATURE_NO_ZERO_PACKETS (1 << 0)
#define CP_LKM_FEATURE_CLONE_MUXED_INTF (1 << 1)

struct cp_lkm_usb_open_intf
{
    int unique_id;
};

struct cp_lkm_usb_close_intf
{
    int unique_id;
};

struct cp_lkm_usb_unplug_intf
{
    int unique_id;
};

struct cp_lkm_usb_is_alive_intf
{
    int unique_id;
};

typedef enum
{
    EP_ACTION_NONE = 0,
    EP_ACTION_CREATE,
    EP_ACTION_LISTEN,
    EP_ACTION_LISTEN_STOP,
    EP_ACTION_RECV,
	EP_ACTION_FLUSH_CONTROL,
	EP_ACTION_SET_MAX_TX_SIZE
}cp_lkm_usb_ep_action_t;

struct cp_lkm_usb_ep_action
{
    int unique_id;
    cp_lkm_usb_ep_action_t action;
    int ep_num; 
    int ep_type;             //UE_BULK or UE_INTERRUPT
    int interval;            //interrupt end points
    int max_transfer_size;
};

struct cp_lkm_usb_pm_link
{
	int usb_unique_id; 	// usb device to link or unlink
	int pm_unique_id; 	// pm to link or unlink
	int link; 			// link or unlink the usb device and pm
};

struct cp_lkm_usb_set_wrapper
{
	int unique_id; 	// usb device to link or unlink
	cp_lkm_wrapper_t wrapper;
	void* wrapper_info;             //pointer to a chunk of something, must be copied out by shim
	int wrapper_info_len;
	int clone_num;
	int mux_id;
};

struct cp_lkm_usb_set_mux_id
{
	int unique_id; 	
	int clone_num;
	int mux_id;
};

#endif
